@@ -84,6 +84,7 @@ typedef struct {
 	FLAC__int32 **align_reservoir;
 	unsigned *align_reservoir_samples;
 	FLAC__bool replay_gain;
+	FLAC__bool ignore_chunk_sizes;
 	FLAC__bool sector_align;
 
 	FLAC__StreamMetadata *vorbis_comment;
@@ -94,6 +95,7 @@ typedef struct {
 		FLAC__bool disable_constant_subframes;
 		FLAC__bool disable_fixed_subframes;
 		FLAC__bool disable_verbatim_subframes;
+		FLAC__bool do_md5;
 	} debug;
 } encode_options_t;
 
