@@ -1,5 +1,5 @@
 /* libFLAC - Free Lossless Audio Codec library
- * Copyright (C) 2001,2002,2003,2004,2005  Josh Coalson
+ * Copyright (C) 2001,2002,2003,2004,2005,2006,2007  Josh Coalson
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -29,13 +29,13 @@
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
+#if HAVE_CONFIG_H
+#  include <config.h>
+#endif
+
 #include "private/memory.h"
 #include "FLAC/assert.h"
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 void *FLAC__memory_alloc_aligned(size_t bytes, void **aligned_address)
 {
 	void *x;
@@ -45,7 +45,24 @@ void *FLAC__memory_alloc_aligned(size_t bytes, void **aligned_address)
 #ifdef FLAC__ALIGN_MALLOC_DATA
 	/* align on 32-byte (256-bit) boundary */
 	x = malloc(bytes+31);
-	*aligned_address = (void*)(((unsigned)x + 31) & -32);
+#ifdef SIZEOF_VOIDP
+#if SIZEOF_VOIDP == 4
+		/* could do  *aligned_address = x + ((unsigned) (32 - (((unsigned)x) & 31))) & 31; */
+		*aligned_address = (void*)(((unsigned)x + 31) & -32);
+#elif SIZEOF_VOIDP == 8
+		*aligned_address = (void*)(((FLAC__uint64)x + 31) & (FLAC__uint64)(-((FLAC__int64)32)));
+#else
+# error  Unsupported sizeof(void*)
+#endif
+#else
+	/* there's got to be a better way to do this right for all archs */
+	if(sizeof(void*) == sizeof(unsigned))
+		*aligned_address = (void*)(((unsigned)x + 31) & -32);
+	else if(sizeof(void*) == sizeof(FLAC__uint64))
+		*aligned_address = (void*)(((FLAC__uint64)x + 31) & (FLAC__uint64)(-((FLAC__int64)32)));
+	else
+		return 0;
+#endif
 #else
 	x = malloc(bytes);
 	*aligned_address = x;
